-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "DS_InstantLimb"

-- **************************************************
-- General information about this script
-- **************************************************

DS_InstantLimb = {}
function DS_InstantLimb:Name()
	return "Instant Limb"
end

function DS_InstantLimb:Version()
	return "0.9"
end

function DS_InstantLimb:Description()
	return "Creates a limb ready to animate."
end

function DS_InstantLimb:Creator()
	return "dauid.com"
end

function DS_InstantLimb:UILabel()
	return("Instant Limb")
end
-- **************************************************
-- Recurring values
-- **************************************************
DS_InstantLimb.parentbone = nil						--The parent of the selected bone
DS_InstantLimb.lowerbone = nil						--The selected bone
DS_InstantLimb.anglebone = nil						--The bone that will be added at the joint
DS_InstantLimb.upperAngle =0 						--This will store the angle of the upper bone
DS_InstantLimb.middleAngle = 0 						--This will store the angle of the added bone, i.e half the lower bone's angle
DS_InstantLimb.lowerAngle=0 						--This will store the angle of the lower bone
DS_InstantLimb.upperPos = LM.Vector2:new_local()	--Position of the upper bone
DS_InstantLimb.lowerPos = LM.Vector2:new_local()	--Position of the lower bone
DS_InstantLimb.thirdPos = LM.Vector2:new_local()	--The end point of the lower bone
DS_InstantLimb.jointCurvature=0.4406054019928		--Line curvature in the elbow/knee
DS_InstantLimb.upperJoint = LM.Vector2:new_local() 	--Position of the upper cross section of the arm
DS_InstantLimb.lowerJoint = LM.Vector2:new_local()	--Position of the lower cross section of the arm
DS_InstantLimb.bentWrongWay=false					--If the lower bone is rotated clockwise at an angle greater than the upper bone then the points in the very middle of the arm have to be flipped.
DS_InstantLimb.lowerShapeOnTop=true					--True for making an arm (the lower arm will overlap the upper arm. False for making a leg ( not necessarily but I think it looks best).
DS_InstantLimb.lineWidth=0.0055555556900799			--The line width and also radius of the gap filling circles
DS_InstantLimb.fillGaps=true						--If true will create circles to fill line gaps in joints
DS_InstantLimb.upperRadius=0.15						--Radius of shoulder
DS_InstantLimb.lowerRadius=0.15						--Radius of elbow
DS_InstantLimb.bottomRadius=0.15					--Radius of wrist
DS_InstantLimb.theDistance = 0.025					--The distance between cross sections in the arm
DS_InstantLimb.lineWidthInMoho=0					--For calculating line width

-- **************************************************
-- Options dialog
-- **************************************************

local DS_InstantLimbDialog = {}

DS_InstantLimbDialog.UPDATE = MOHO.MSG_BASE
DS_InstantLimbDialog.LOWERSHAPE = MOHO.MSG_BASE+1
DS_InstantLimbDialog.UPPERSHAPE = MOHO.MSG_BASE+2
DS_InstantLimbDialog.RESET = MOHO.MSG_BASE+3

function DS_InstantLimbDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Instant limb options", DS_InstantLimbDialog)
	local l = d:GetLayout()

	d.moho = moho
		if (moho.document:Height()>moho.document:Width()) then
		DS_InstantLimb.lineWidthInMoho=moho.document:Width()
		else
		DS_InstantLimb.lineWidthInMoho=moho.document:Height()
		end
	local aspect=moho.document:AspectRatio()
	local previewY= 300
	local previewX=previewY*moho.document:AspectRatio()
	
			l:AddChild(LM.GUI.StaticText("NOTE: Final shape will not have holes and lines at joint."), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Bones have to be inside the workspace for shape to show up in preview window."), LM.GUI.ALIGN_LEFT)
	
	d.preview = MOHO.MeshPreview(previewX, previewY)
	l:AddChild(d.preview)
l:PushH()
	l:PushV()
		l:AddChild(LM.GUI.StaticText("Upper radius:"), LM.GUI.ALIGN_LEFT)
		l:AddChild(LM.GUI.StaticText("Line width:"), LM.GUI.ALIGN_LEFT)
		
	l:Pop()
	l:PushV()
		d.upperRadius = LM.GUI.TextControl(0, "0.0000", DS_InstantLimbDialog.UPDATE, LM.GUI.FIELD_UFLOAT)
		d.upperRadius:SetConstantMessages(true)
		d.upperRadius:SetWheelInc(0.01)
		l:AddChild(d.upperRadius)
		
		d.lineWidth = LM.GUI.TextControl(0, "0.0000", DS_InstantLimbDialog.UPDATE, LM.GUI.FIELD_UFLOAT)
			d.lineWidth:SetValue(DS_InstantLimb.lineWidth*DS_InstantLimb.lineWidthInMoho)
			d.lineWidth:SetConstantMessages(true)
			d.lineWidth:SetWheelInc(1)
			l:AddChild(d.lineWidth)
	l:Pop()
	l:PushV()
		l:AddChild(LM.GUI.StaticText("Middle radius:"), LM.GUI.ALIGN_LEFT)
		l:AddChild(LM.GUI.StaticText("Joint curvature:"), LM.GUI.ALIGN_LEFT)
	l:Pop()
	l:PushV()	
			
			
			d.middleRadius = LM.GUI.TextControl(0, "0.0000", DS_InstantLimbDialog.UPDATE, LM.GUI.FIELD_UFLOAT)
			d.middleRadius:SetConstantMessages(true)
			d.middleRadius:SetWheelInc(0.01)
			l:AddChild(d.middleRadius)
			
			d.jointCurvature = LM.GUI.TextControl(0, "0.0000", DS_InstantLimbDialog.UPDATE, LM.GUI.FIELD_UFLOAT)
			d.jointCurvature:SetConstantMessages(true)
			d.jointCurvature:SetWheelInc(0.05)
			l:AddChild(d.jointCurvature)
			
			
	l:Pop()
	l:PushV()
		l:AddChild(LM.GUI.StaticText("Lower radius:"), LM.GUI.ALIGN_LEFT)
	l:Pop()

	l:PushV()
	
		d.lowerRadius = LM.GUI.TextControl(0, "0.0000", DS_InstantLimbDialog.UPDATE, LM.GUI.FIELD_UFLOAT)
		d.lowerRadius:SetConstantMessages(true)
		d.lowerRadius:SetWheelInc(0.01)
		l:AddChild(d.lowerRadius)
	
	l:Pop()
l:Pop()
	l:PushH()

			l:AddChild(LM.GUI.StaticText("Distance between joint cross sections:"), LM.GUI.ALIGN_LEFT)
			d.theDistance = LM.GUI.TextControl(0, "0.0000", DS_InstantLimbDialog.UPDATE, LM.GUI.FIELD_UFLOAT)
			d.theDistance:SetConstantMessages(true)
			d.theDistance:SetWheelInc(0.01)
			l:AddChild(d.theDistance)

	l:Pop()
	l:PushH()
	l:AddChild(LM.GUI.Divider(false))
	l:Pop()
	l:AddChild(LM.GUI.StaticText("Options not visible in preview:"), LM.GUI.ALIGN_LEFT)
	
	l:PushH()
		l:PushV()
	d.lowerShapeOnTop=LM.GUI.RadioButton("Lower shape on top (arm)", DS_InstantLimbDialog.LOWERSHAPE)
	l:AddChild(d.lowerShapeOnTop, LM.GUI.ALIGN_LEFT)
	
	d.upperShapeOnTop=LM.GUI.RadioButton("Upper shape on top (leg)", DS_InstantLimbDialog.UPPERSHAPE)
	l:AddChild(d.upperShapeOnTop, LM.GUI.ALIGN_LEFT)
	
	if (DS_InstantLimb.lowerShapeOnTop==true) then
	d.lowerShapeOnTop:SetValue(true)
	d.upperShapeOnTop:SetValue(false)
	else
	d.lowerShapeOnTop:SetValue(false)
	d.upperShapeOnTop:SetValue(true)
	end
	
	l:Pop()
	l:PushV()
	
	l:AddChild(LM.GUI.Divider(true))
	l:Pop()
	l:PushV()
	d.fillGaps = LM.GUI.CheckBox("Fill gaps in joint")
	d.fillGaps:SetValue(DS_InstantLimb.fillGaps)
	l:AddChild(d.fillGaps, LM.GUI.ALIGN_LEFT)
	l:Pop()
	l:Pop()
	l:PushH()
	d.reset=LM.GUI.Button("Reset settings", DS_InstantLimbDialog.RESET)
	l:AddChild(d.reset, LM.GUI.ALIGN_RIGHT)

	return d
end

function DS_InstantLimbDialog:UpdateWidgets()
	self.upperRadius:SetValue(DS_InstantLimb.upperRadius)
	self.middleRadius:SetValue(DS_InstantLimb.lowerRadius)
	self.lowerRadius:SetValue(DS_InstantLimb.bottomRadius)
	self.theDistance:SetValue(DS_InstantLimb.theDistance)
	self.lineWidth:SetValue(DS_InstantLimb.lineWidth*DS_InstantLimb.lineWidthInMoho)
	self.jointCurvature:SetValue(DS_InstantLimb.jointCurvature)
	self:UpdatePreview()
end

function DS_InstantLimbDialog:HandleMessage(what)
	if (what == self.UPDATE) then

	
		self:UpdatePreview()
	end
	if (what == self.LOWERSHAPE) then
		self.lowerShapeOnTop:SetValue(true)
		self.upperShapeOnTop:SetValue(false)
	end
	if (what == self.UPPERSHAPE) then
		self.lowerShapeOnTop:SetValue(false)
		self.upperShapeOnTop:SetValue(true)
	end
	if (what == self.RESET) then
	DS_InstantLimb.jointCurvature=0.4406054019928						
	DS_InstantLimb.lowerShapeOnTop=true					
	DS_InstantLimb.lineWidth=4/DS_InstantLimb.lineWidthInMoho
	DS_InstantLimb.fillGaps=true						
	DS_InstantLimb.upperRadius=0.15						
	DS_InstantLimb.lowerRadius=0.15	
	DS_InstantLimb.bottomRadius=0.15
	DS_InstantLimb.theDistance = 0.025
	self.lowerShapeOnTop:SetValue(true)
	self.upperShapeOnTop:SetValue(false)
	self.fillGaps:SetValue(true)
	self:UpdateWidgets()
	self:UpdatePreview()
	end
end

function DS_InstantLimbDialog:OnOK()
	DS_InstantLimb.upperRadius = self.upperRadius:FloatValue()
	DS_InstantLimb.lowerRadius = self.middleRadius:FloatValue()
	DS_InstantLimb.bottomRadius = self.lowerRadius:FloatValue()
	DS_InstantLimb.theDistance = self.theDistance:FloatValue()
	DS_InstantLimb.lowerShapeOnTop = self.lowerShapeOnTop:Value()
	DS_InstantLimb.fillGaps =self.fillGaps:Value()
	DS_InstantLimb.lineWidth =self.lineWidth:FloatValue()/DS_InstantLimb.lineWidthInMoho
	DS_InstantLimb.jointCurvature = self.jointCurvature:FloatValue()
end

function DS_InstantLimbDialog:UpdatePreview()
	local mesh = self.preview:Mesh()
	mesh:Clear()
	
	DS_InstantLimb.theDistance = self.theDistance:FloatValue()
	DS_InstantLimb.upperRadius = self.upperRadius:FloatValue()
	DS_InstantLimb.lowerRadius = self.middleRadius:FloatValue()
	DS_InstantLimb.bottomRadius = self.lowerRadius:FloatValue()
	DS_InstantLimb.lineWidth=self.lineWidth:FloatValue()/DS_InstantLimb.lineWidthInMoho
	DS_InstantLimb.jointCurvature = self.jointCurvature:FloatValue()
	

	DS_InstantLimb:buildMesh(mesh)
	self.preview:CreateShape(true)
	mesh:Shape(0).fMyStyle.fLineWidth=DS_InstantLimb.lineWidth
	
	self.preview:Refresh()
end

-- **************************************************
-- The guts of this script
-- **************************************************


function DS_InstantLimb:Run(moho)

if (moho.layer:LayerType() ~= MOHO.LT_VECTOR) then
		LM.GUI.Alert(LM.GUI.ALERT_INFO, "No vector layer selected", "Please select or create a vector layer inside a bone layer.", nil, "OK", nil, nil)
		return
end

local skel = moho:ParentSkeleton()
	if (skel == nil) then
		LM.GUI.Alert(LM.GUI.ALERT_INFO, "No parent skeleton found.", nil, nil, "OK", nil, nil)
		return
	end

for i = 0, skel:CountBones() - 1 do --Go through every bone
		if (skel:Bone(i).fSelected) then --If the current bone is selected
		local tempb=skel:Bone(i).fParent
			if (tempb<0) then
				LM.GUI.Alert(LM.GUI.ALERT_INFO, "Bone does not have a parent.", "Please select a child bone.", nil, "OK", nil, nil)
				return
			end
		end
end



moho.document:PrepUndo(moho.layer)
moho.document:SetDirty()
-------------------------------------Get info about bone positions and angles
self.bentWrongWay=false
self.upperAngle=0
self.upperPos:Set(0, 0)
	
	for lower = 0, skel:CountBones() - 1 do --Go through every bone
	 self.lowerbone = skel:Bone(lower)
		if (self.lowerbone.fSelected) then --If the current bone is selected
			break
			end
	end
			
	if (self.lowerbone:Name()=="") then
	self.lowerbone:SetName("Random Name"..math.random(9999))
	end
		
	self.parentbone=skel:Bone(self.lowerbone.fParent) --Get parent bone of selected bone

--------Bunch of  code to get the positions and rotations of any parent bones to the upper bone in the limb (Big thanks to HeyVern at the Lost Marble forum)
		if (self.parentbone.fParent>-1) then 
		
            local bParent = self.lowerbone.fParent 
            while bParent >-1 do 
                local bone = skel:Bone(bParent) 
                self.upperAngle = self.upperAngle + bone.fAngle 
                bParent = bone.fParent 
            end 
            self.parentbone.fRestMatrix:Transform(self.upperPos)
		else
		self.upperPos=self.parentbone.fPos
		self.upperAngle=self.parentbone.fAngle
		end
---------------------------------------------------------------------------------------------------------------------------------

		if (self.lowerbone.fAngle>math.pi)then
		self.bentWrongWay=true
		end
			self.lowerAngle = self.upperAngle+self.lowerbone.fAngle --Get the angle
			self.middleAngle = self.upperAngle+(self.lowerbone.fAngle*0.5)
			self.lowerPos=self.parentbone.fPos+self.lowerbone.fPos 
			self.lowerPos.x=self.upperPos.x+math.cos(self.upperAngle)*(self.lowerbone.fPos.x) --Get the position of what I call middle joint
			self.lowerPos.y=self.upperPos.y+math.sin(self.upperAngle)*(self.lowerbone.fPos.x)
	
			self.thirdPos.x=self.lowerPos.x+math.cos(self.lowerAngle)*(self.lowerbone.fLength)
			self.thirdPos.y=self.lowerPos.y+math.sin(self.lowerAngle)*(self.lowerbone.fLength)
			
-------------------------------------End of info gathering

local dlog = DS_InstantLimbDialog:new(moho) --Show options dialog
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end
self:BuildLimb(moho)
end

function DS_InstantLimb:buildMesh(mesh)
local v = LM.Vector2:new_local()
local oldNumPoints = mesh:CountPoints()

self.upperJoint.x=self.upperPos.x+math.cos(self.upperAngle)*(self.lowerbone.fPos.x-self.theDistance) --Get the position of what I call the upper joint
self.upperJoint.y=self.upperPos.y+math.sin(self.upperAngle)*(self.lowerbone.fPos.x-self.theDistance)
		
self.lowerJoint.x=self.lowerPos.x+math.cos(self.lowerAngle)*(self.theDistance) --then get the poition offset on the self.lowerbone. Hard to explain
self.lowerJoint.y=self.lowerPos.y+math.sin(self.lowerAngle)*(self.theDistance)

v.x=(self.upperPos.x)+(math.cos(self.upperAngle+(math.pi/2))*self.upperRadius)
v.y=(self.upperPos.y)+(math.sin(self.upperAngle+(math.pi/2))*self.upperRadius)
local pointOne=(mesh:AddLonePoint(v, 0))
v.x=(self.upperPos.x)+(math.cos(self.upperAngle+math.pi)*self.upperRadius)
v.y=(self.upperPos.y)+(math.sin(self.upperAngle+math.pi)*self.upperRadius)
local pointTwo=(mesh:AppendPoint(v, 0))
v.x=(self.upperPos.x)+(math.cos(self.upperAngle+(math.pi*1.5))*self.upperRadius)
v.y=(self.upperPos.y)+(math.sin(self.upperAngle+(math.pi*1.5))*self.upperRadius)
local pointThree=(mesh:AppendPoint(v, 0))

v.x=(self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi*1.5))*self.lowerRadius)
v.y=(self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi*1.5))*self.lowerRadius)
local pointFour=(mesh:AppendPoint(v, 0))
----------------------------------------------------------------------------------------
	if (self.bentWrongWay) then
		v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius)
		v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius)
	else
		v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
		v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	end
local pointFive=(mesh:AppendPoint(v, 0))
----------------------------------------------------------------------------------------
v.x=(self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi*1.5))*self.lowerRadius)
v.y=(self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi*1.5))*self.lowerRadius)
local pointSix=(mesh:AppendPoint(v, 0))

v.x=(self.thirdPos.x)+(math.cos(self.lowerAngle+(math.pi*1.5))*self.bottomRadius)
v.y=(self.thirdPos.y)+(math.sin(self.lowerAngle+(math.pi*1.5))*self.bottomRadius)
local pointSeven=(mesh:AppendPoint(v, 0))

v.x=self.thirdPos.x
v.y=self.thirdPos.y
local pointEight=(mesh:AppendPoint(v, 0))

v.x=(self.thirdPos.x)+(math.cos(self.lowerAngle+(math.pi/2))*self.bottomRadius)
v.y=(self.thirdPos.y)+(math.sin(self.lowerAngle+(math.pi/2))*self.bottomRadius)
local pointNine=(mesh:AppendPoint(v, 0))

v.x=(self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi/2))*self.lowerRadius)
v.y=(self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi/2))*self.lowerRadius)
local pointTen=(mesh:AppendPoint(v, 0))
---------------------------------------------------------------------------------------
	if (self.bentWrongWay) then
		v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
		v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	else
		v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius)
		v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius)
	end
local pointEleven=(mesh:AppendPoint(v, 0))
---------------------------------------------------------------------------------------
v.x=(self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi/2))*self.lowerRadius)
v.y=(self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi/2))*self.lowerRadius)
local pointTwelve=(mesh:AppendPoint(v, 0))

v.x=(self.upperPos.x)+(math.cos(self.upperAngle+(math.pi/2))*self.upperRadius)
v.y=(self.upperPos.y)+(math.sin(self.upperAngle+(math.pi/2))*self.upperRadius)
local pointThirteen=(mesh:AppendPoint(v, 0))

mesh:WeldPoints(oldNumPoints, oldNumPoints+12, 0) --Weld the big limb shape
--------------------------------------------------------------------------------------------- Create upper cross section
mesh:SelectNone()
v.x=(self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi*1.5))*self.lowerRadius)
v.y=(self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi*1.5))*self.lowerRadius)
local pointFourteen=(mesh:AddLonePoint(v, 0))

v.x=(self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi/2))*self.lowerRadius)
v.y=(self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi/2))*self.lowerRadius)
local pointFifteen=(mesh:AppendPoint(v, 0))
mesh:SelectNone()

mesh:WeldPoints(oldNumPoints+2, oldNumPoints+12, 0) --Weld the upper cross section
mesh:WeldPoints(oldNumPoints+12, oldNumPoints+9, 0) --Weld the upper cross section
--------------------------------------------------------------------------------------------- Create middle cross section
mesh:SelectNone()
	if (self.bentWrongWay) then
	v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius)
	v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius)
	else
	v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	end
local pointSixteen=(mesh:AddLonePoint(v, 0))
	
	if (self.bentWrongWay) then
	v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius)
	else
	v.x=(self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius)
	v.y=(self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius)
	end

local pointSeventeen=(mesh:AppendPoint(v, 0))

mesh:WeldPoints(oldNumPoints+2, oldNumPoints+12, 0) --Weld the middle cross section
mesh:WeldPoints(oldNumPoints+12, oldNumPoints+7, 0) --Weld the middle cross section
--------------------------------------------------------------------------------------------- Create middle cross section
mesh:SelectNone()
v.x=(self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi*1.5))*self.lowerRadius)
v.y=(self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi*1.5))*self.lowerRadius)
local pointEighteen=(mesh:AddLonePoint(v, 0))

v.x=(self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi/2))*self.lowerRadius)
v.y=(self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi/2))*self.lowerRadius)
local pointNineteen=(mesh:AppendPoint(v, 0))

mesh:WeldPoints(oldNumPoints+2, oldNumPoints+12, 0) --Weld the lower cross section
mesh:WeldPoints(oldNumPoints+12, oldNumPoints+5, 0) --Weld the lower cross section
--------------------------------------------------------------------------------------------- Set the curvature
local peakedTable= {oldNumPoints+1, oldNumPoints+2, oldNumPoints+4, oldNumPoints+5, oldNumPoints+7, oldNumPoints+8, oldNumPoints+9, oldNumPoints+11}
for i, v in ipairs(peakedTable) do
	mesh:Point(v):SetCurvature(MOHO.PEAKED, 0)
end

mesh:Point(oldNumPoints):SetCurvature(0.65895956754684, 0)
mesh:Point(oldNumPoints+3):SetCurvature(0.65895956754684, 0)

mesh:Point(oldNumPoints+6):SetCurvature(self.jointCurvature, 0)
mesh:Point(oldNumPoints+10):SetCurvature(self.jointCurvature, 0)

mesh:SelectAll()



end
------------------------------------------------------End of buildMesh
function DS_InstantLimb:BuildLimb(moho)
local skel = moho:ParentSkeleton()
	if (skel == nil) then
		return
	end

	
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end
------------------------------------------------------------------------------------------------ Add the extra bone
self.angleBone=skel:AddBone(0)
local NumBones=skel:CountBones()-1
skel:Bone(NumBones).fParent=skel:BoneID(self.parentbone)
skel:Bone(NumBones).fAnimPos:SetValue(0, self.lowerbone.fPos)
skel:Bone(NumBones).fAnimAngle:SetValue(0, (math.pi*1.5)+(self.lowerbone.fAngle/2))
skel:Bone(NumBones).fLength=self.lowerRadius
skel:Bone(NumBones).fAngleControlParent=skel:BoneID(self.lowerbone)
skel:Bone(NumBones).fAngleControlScale=0.5
------------------------------------------------------------------------------------------------ Start building
mesh:SelectNone()
local oldNumPoints = mesh:CountPoints()
local oldNumCurves = mesh:CountCurves()
DS_InstantLimb:buildMesh(mesh)
mesh:SelectNone()
--------------------------------------------------------------------------------------------- Hide edges
mesh:Curve(oldNumCurves+1):SetSegmentOn(0, false)
mesh:Curve(oldNumCurves+2):SetSegmentOn(0, false)
mesh:Curve(oldNumCurves+3):SetSegmentOn(0, false)
--------------------------------------------------------------------------------------------- Fill gaps with circles
if (self.fillGaps) then
	if (self.bentWrongWay) then
			self:fillGapFunction(mesh, (self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius), (self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius), self.lineWidth)
			self:fillGapFunction(mesh, (self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius), (self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius), self.lineWidth)
		else
			self:fillGapFunction(mesh, (self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi*1.5))*self.lowerRadius), (self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi*1.5))*self.lowerRadius), self.lineWidth)
			self:fillGapFunction(mesh, (self.lowerPos.x)+(math.cos(self.middleAngle+(math.pi/2))*self.lowerRadius), (self.lowerPos.y)+(math.sin(self.middleAngle+(math.pi/2))*self.lowerRadius), self.lineWidth)
		end

self:fillGapFunction(mesh, (self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi*1.5))*self.lowerRadius), (self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi*1.5))*self.lowerRadius), self.lineWidth)
self:fillGapFunction(mesh, (self.upperJoint.x)+(math.cos(self.upperAngle+(math.pi/2))*self.lowerRadius), (self.upperJoint.y)+(math.sin(self.upperAngle+(math.pi/2))*self.lowerRadius), self.lineWidth)
self:fillGapFunction(mesh, (self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi*1.5))*self.lowerRadius), (self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi*1.5))*self.lowerRadius), self.lineWidth)
self:fillGapFunction(mesh, (self.lowerJoint.x)+(math.cos(self.lowerAngle+(math.pi/2))*self.lowerRadius), (self.lowerJoint.y)+(math.sin(self.lowerAngle+(math.pi/2))*self.lowerRadius), self.lineWidth)
end
--------------------------------------------------------------------------------------------- Start making shapes
local upperShape={oldNumPoints, oldNumPoints+1, oldNumPoints+7, oldNumPoints+8, oldNumPoints+9}
local upperMiddleShape = {oldNumPoints+6, oldNumPoints+7, oldNumPoints+9, oldNumPoints+10}
local lowerMiddleShape = {oldNumPoints+5, oldNumPoints+6, oldNumPoints+10, oldNumPoints+11}
local lowerShape = {oldNumPoints+2, oldNumPoints+3, oldNumPoints+4, oldNumPoints+5, oldNumPoints+11}
local topCircles = {oldNumPoints+20, oldNumPoints+21, oldNumPoints+22, oldNumPoints+23, oldNumPoints+24, oldNumPoints+25, oldNumPoints+26, oldNumPoints+27}
local middleCircles = {oldNumPoints+12, oldNumPoints+13, oldNumPoints+14, oldNumPoints+15, oldNumPoints+16, oldNumPoints+17, oldNumPoints+18, oldNumPoints+19}
local bottomCircles = {oldNumPoints+28, oldNumPoints+29, oldNumPoints+30, oldNumPoints+31, oldNumPoints+32, oldNumPoints+33, oldNumPoints+34, oldNumPoints+35}

	if (self.lowerShapeOnTop) then
	
		for i, v in ipairs(upperShape) do
			mesh:Point(v).fSelected=true
		end
		local theUpperShape=moho:CreateShape(true)
		mesh:Shape(theUpperShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		for i, v in ipairs(upperMiddleShape) do
			mesh:Point(v).fSelected=true
		end
		local theUpperMiddleShape=moho:CreateShape(true)
		mesh:Shape(theUpperMiddleShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(topCircles) do
			mesh:Point(v).fSelected=true
			end
			local theTopCircles=moho:CreateShape(true)
			
			mesh:Shape(theTopCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theTopCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theTopCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			
			mesh:SelectNone()
		end
		
		for i, v in ipairs(lowerMiddleShape) do
			mesh:Point(v).fSelected=true
		end
		local theLowerMiddleShape=moho:CreateShape(true)
		mesh:Shape(theLowerMiddleShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(middleCircles) do
			mesh:Point(v).fSelected=true
			end
			local theMiddleCircles=moho:CreateShape(true)
			
			mesh:Shape(theMiddleCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theMiddleCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theMiddleCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			mesh:SelectNone()
		end
		
		for i, v in ipairs(lowerShape) do
			mesh:Point(v).fSelected=true
		end
		local theLowerShape=moho:CreateShape(true)
		mesh:Shape(theLowerShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(bottomCircles) do
			mesh:Point(v).fSelected=true
			end
			local theBottomCircles=moho:CreateShape(true)
			
			mesh:Shape(theBottomCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theBottomCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theBottomCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			
			mesh:SelectNone()
		end
		
	else
	
		for i, v in ipairs(lowerShape) do
				mesh:Point(v).fSelected=true
			end
			local theLowerShape=moho:CreateShape(true)
			mesh:Shape(theLowerShape).fMyStyle.fLineWidth=self.lineWidth
			mesh:SelectNone()
			
		for i, v in ipairs(lowerMiddleShape) do
			mesh:Point(v).fSelected=true
		end
		local theLowerMiddleShape=moho:CreateShape(true)
		mesh:Shape(theLowerMiddleShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(bottomCircles) do
			mesh:Point(v).fSelected=true
			end
			local theBottomCircles=moho:CreateShape(true)
			
			mesh:Shape(theBottomCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theBottomCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theBottomCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			
			mesh:SelectNone()
		end
		
		for i, v in ipairs(upperMiddleShape) do
			mesh:Point(v).fSelected=true
		end
		local theUpperMiddleShape=moho:CreateShape(true)
		mesh:Shape(theUpperMiddleShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(middleCircles) do
			mesh:Point(v).fSelected=true
			end
			local theMiddleCircles=moho:CreateShape(true)
			
			mesh:Shape(theMiddleCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theMiddleCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theMiddleCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			mesh:SelectNone()
		end
		
		for i, v in ipairs(upperShape) do
			mesh:Point(v).fSelected=true
		end
		local theUpperShape=moho:CreateShape(true)
		mesh:Shape(theUpperShape).fMyStyle.fLineWidth=self.lineWidth
		mesh:SelectNone()
		
		if (self.fillGaps) then
			for i, v in ipairs(topCircles) do
			mesh:Point(v).fSelected=true
			end
			local theTopCircles=moho:CreateShape(true)
			
			mesh:Shape(theTopCircles).fHasOutline = false
			local strokeColor = LM.ColorVector:new_local() 
			StrokeColor=mesh:Shape(theTopCircles).fMyStyle.fLineCol:GetValue(0)
			mesh:Shape(theTopCircles).fMyStyle.fFillCol:SetValue(0, StrokeColor)
			
			mesh:SelectNone()
		end
	end
--------------------------------------------------------------------------------------------- Bind to bones
local upperBoneBinding = {oldNumPoints, oldNumPoints+1, oldNumPoints+7, oldNumPoints+8, oldNumPoints+9}

		for i, v in ipairs(upperBoneBinding) do
			mesh:Point(v).fParent=(skel:BoneID(self.parentbone))
		end

local middleBoneBinding = {oldNumPoints+6, oldNumPoints+10}

		for i, v in ipairs(middleBoneBinding) do
			mesh:Point(v).fParent=(NumBones)
		end
local lowerboneBinding = {oldNumPoints+2, oldNumPoints+3, oldNumPoints+4, oldNumPoints+5, oldNumPoints+11}

		for i, v in ipairs(lowerboneBinding) do
			mesh:Point(v).fParent=(skel:BoneID(self.lowerbone))
		end
		
if (self.fillGaps) then
	for i, v in ipairs(topCircles) do
			mesh:Point(v).fParent=(skel:BoneID(self.parentbone))
	end
	
	for i, v in ipairs(middleCircles) do
			mesh:Point(v).fParent=(NumBones)
	end
	
	for i, v in ipairs(bottomCircles) do
			mesh:Point(v).fParent=(skel:BoneID(self.lowerbone))
	end
end
-------------------------------------------------------------------------------
	moho.view:DrawMe()
end

function DS_InstantLimb:fillGapFunction(mesh, theX, theY, size)


local oldNumPoints = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	mesh:SelectNone()
	
	
	v.x=theX-size
	v.y=theY
local pointOne=(mesh:AddLonePoint(v, 0))
	v.x=theX
	v.y=theY+size
local pointTwo=(mesh:AppendPoint(v, 0))

	v.x=theX+size
	v.y=theY
local pointThree=(mesh:AppendPoint(v, 0))

	v.x=theX
	v.y=theY-size
local pointFour=(mesh:AppendPoint(v, 0))

	v.x=theX-size
	v.y=theY
local pointFive=(mesh:AppendPoint(v, 0))

mesh:WeldPoints(oldNumPoints, oldNumPoints+4, 0)
	
	for i = 0, 3 do
		mesh:Point(oldNumPoints+i):SetCurvature(0.391379, 0)
		mesh:Point(oldNumPoints+i).fSelected=true
		
	end
	mesh:SelectNone()
end